REM programming a PIC16F84 with QBasic/FirstBasic
REM by Terry Newton - Aug 26, 1999 - Sept 27, 1999
REM Modified Nov 18, 2006 to allow specifying port

compiled = 1    ' make 1 if compiled, 0 if running under qbasic
                ' (1 = read cl, use high timing constants)
doprogram = 1
doeeprom = 1
doconfig = 1
noverify = 0
nohwcheck = 0
dump = 0

' timing constants
cmddelay& = 50
tenmsdelay& = 1000     ' starting value, self calibrating
ackdelay& = 30
portdelay& = 30

IF compiled = 0 THEN
 cmddelay& = 1
 tenmsdelay& = 250
 ackdelay& = 1
 portdelay& = 1
END IF

DIM program%(1024) 'program memory
DIM eedata%(64)    'eeprom data
DIM config%(8)     'config codes 0-3=id 7=config
pport1 = &H378
pport2 = &H278
portadr = pport1  'default port 1

' programming codes
loadconfig = 0
loaddata = 2
readdata = 4
incaddr = 6
beginprog = 8
loaddatadm = 3
readdatadm = 5
programmode = 65

'port pins...
ppdata = 1     'bit 0  pic data pin rb7
ppclock = 2    'bit 1  pic clock pin rb6
ppvpp = 4      'bit 2  1 = mclr 0, 0 = mclr vpp
pphigh = 8     'bit 3  make 1 to read data through ack

nocl = 1: nofilename = 1: ON ERROR GOTO generalerror
PRINT : PRINT "  *** Newton's PIC16x84 Burner ***"
IF compiled THEN GOSUB getparms

GOSUB calibratedelay  ' verify/calibrate 10ms delay
FOR i = 1 TO 5        ' make sure windows wasn't just fooling
 GOSUB delay10ms: GOSUB delay10ms
 GOSUB calibratedelay
NEXT i

statadr = portadr + 1
IF nohwcheck = 0 THEN
 GOSUB testhardware        'check if programmer is attached
 IF hardwarefault THEN
  portadr = pport1: statadr = portadr + 1
  GOSUB testhardware
  IF hardwarefault THEN
   portadr = pport2: statadr = portadr + 1
   GOSUB testhardware
   IF hardwarefault THEN
    PRINT "  Programming hardware not detected"
    GOTO errorquit
   END IF
  END IF
 END IF
END IF

IF nocl THEN
 PRINT "  (P) Program  (D) Dump  (I) Info  (Q) Quit  "
 PRINT "  Select: ";
 gotkey = 0: dump = 0: info = 0
 WHILE gotkey = 0
  GOSUB getkey
  IF ke$ = "q" THEN gotkey = 1: quitprog = 1
  IF ke$ = "d" THEN gotkey = 1: dump = 1
  IF ke$ = "i" THEN gotkey = 1: info = 1
  IF ke$ = "p" THEN gotkey = 1
 WEND
 PRINT UCASE$(ke$)
 IF quitprog = 1 GOTO quit
 IF info GOTO help
END IF

IF nofilename THEN
 IF dump THEN
  PRINT "  Enter filename to dump: ";
 ELSE
  PRINT "  Enter filename to program: ";
 END IF
 LINE INPUT f$: f$ = LTRIM$(RTRIM$(f$))
 IF f$ = "" GOTO quit
END IF

IF INSTR(f$, " ") > 0 THEN
 PRINT "  Parameter error"
 GOTO errorquit
END IF

IF dump GOTO dumptofile

GOSUB loadfile
IF loaderror = 0 THEN
 d = config%(7) AND &H3FF0
 IF d <> &H3FF0 THEN
  PRINT "  WARNING! Code protection is enabled, this can prevent"
  PRINT "  the PIC from being reprogrammed. ARE YOU SURE? (Y/N)  ";
  GOSUB getkey: PRINT UCASE$(ke$)
  IF ke$ <> "y" GOTO errorquit
 END IF
 pgmerror = 0
 portdata = ppvpp: GOSUB portout   ' make mclr low (inverted)
 PRINT : PRINT "  Attach programmer to robot"
 PRINT "  Press any key when done, or esc to quit"
 GOSUB getkey
 IF ke$ <> CHR$(27) THEN
  PRINT "  Programming PIC: "; : adrline = CSRLIN
  cmd = programmode: GOSUB dopicstuff
  IF doprogram THEN
   adr = 0: WHILE adr < 1024 AND pgmerror = 0
    dat = program%(adr): sdat = dat
    IF NOT noverify THEN cmd = readdata: GOSUB dopicstuff
    IF noverify OR sdat <> indat THEN
     dat = sdat: GOSUB dispadr
     cmd = loaddata: GOSUB dopicstuff
     cmd = beginprog: GOSUB dopicstuff
    END IF
    IF noverify = 0 THEN
     cmd = readdata: GOSUB dopicstuff
     IF indat <> sdat THEN pgmerror = 1
    END IF
    cmd = incaddr: GOSUB dopicstuff
    adr = adr + 1
   WEND
  END IF
  IF doeeprom THEN
   adr = 0: WHILE adr < 64 AND pgmerror = 0
    dat = eedata%(adr): sdat = dat
    IF noverify = 0 THEN cmd = readdatadm: GOSUB dopicstuff
    IF noverify OR dat <> indat THEN
     dat = sdat: GOSUB dispadr
     cmd = loaddatadm: GOSUB dopicstuff
     cmd = beginprog: GOSUB dopicstuff
    END IF
    IF noverify = 0 THEN
     cmd = readdatadm: GOSUB dopicstuff
     IF indat <> sdat THEN pgmerror = 1
    END IF
    cmd = incaddr: GOSUB dopicstuff
    adr = adr + 1
   WEND
  END IF
  IF doconfig AND pgmerror = 0 THEN
   FOR adr = 0 TO 3
    dat = config%(adr): sdat = dat
    cmd = loadconfig: GOSUB dopicstuff
    IF adr > 0 THEN
     FOR adr1 = 1 TO adr
      cmd = incaddr: GOSUB dopicstuff
     NEXT adr1
    END IF
    IF noverify = 0 THEN cmd = readdata: GOSUB dopicstuff
    IF noverify OR indat <> sdat THEN
     dat = sdat: GOSUB dispadr
     cmd = loadconfig: GOSUB dopicstuff
     IF adr > 0 THEN
      FOR adr1 = 1 TO adr
       cmd = incaddr: GOSUB dopicstuff
      NEXT adr1
     END IF
     cmd = beginprog: GOSUB dopicstuff
     IF noverify = 0 THEN
      cmd = readdata: GOSUB dopicstuff
      IF indat <> sdat THEN pgmerror = 1
     END IF
    END IF
   NEXT adr
   dat = config%(7): sdat = dat
   cmd = loadconfig: GOSUB dopicstuff
   FOR adr1 = 1 TO 7
    cmd = incaddr: GOSUB dopicstuff
   NEXT adr1
   IF noverify = 0 THEN cmd = readdata: GOSUB dopicstuff
   IF noverify OR indat <> sdat THEN
    dat = sdat: adr = 7: GOSUB dispadr
    cmd = loadconfig: GOSUB dopicstuff
    FOR adr1 = 1 TO 7
     cmd = incaddr: GOSUB dopicstuff
    NEXT adr1
    cmd = beginprog: GOSUB dopicstuff
    IF noverify = 0 THEN
     cmd = readdata: GOSUB dopicstuff
     IF indat <> sdat THEN pgmerror = 1
    END IF
   END IF
  END IF
  PRINT : portdata = 0: GOSUB portout
  IF pgmerror THEN
   PRINT "  Verify error - wanted ";
   PRINT HEX$(sdat); " but got "; HEX$(indat)
   GOTO errorquit
  ELSE
   PRINT "  Complete."
  END IF
 END IF
ELSE
 PRINT "  Error loading file "; f$
 GOTO errorquit
END IF

portdata = ppvpp: GOSUB portout   ' leave mclr low (inverted)

quit:
errorquit:
SYSTEM

dispadr:
LOCATE adrline, 19: PRINT adr; "    ";
RETURN

getkey:
ke$ = LCASE$(INKEY$)
IF ke$ = "" GOTO getkey
RETURN

generalerror:
CLOSE
PRINT : PRINT "  An error of some kind has occured"
GOTO errorquit

REM where it all happens
dopicstuff:
IF cmd < 64 THEN
 portdata = 0: GOSUB portout
 scmd = cmd
 FOR i = 1 TO 6
  bit = scmd AND 1: GOSUB output1bit
  scmd = INT(scmd / 2)
 NEXT i
 portdata = 1: GOSUB portout
 IF cmd = incaddr GOTO exitdopic
 IF cmd = beginprog THEN GOSUB delay10ms: GOTO exitdopic
 IF cmd = loaddata OR cmd = loaddatadm OR cmd = loadconfig THEN
  FOR i = 1 TO cmddelay&: NEXT i
  bit = 0: GOSUB output1bit
  FOR i = 1 TO 14
   bit = dat AND 1: GOSUB output1bit
   dat = INT(dat / 2)
  NEXT i
  bit = 0: GOSUB output1bit
  GOTO exitdopic
 END IF
 IF cmd = readdata OR cmd = readdatadm THEN
  portdata = pphigh: GOSUB portout
  FOR z& = 1 TO cmddelay&: NEXT z&
  portdata = pphigh + ppclock: GOSUB portout
  portdata = pphigh: GOSUB portout
  indat = 0
  FOR i = 1 TO 14
   indat = INT(indat / 2)
   portdata = pphigh + ppclock: GOSUB portout
   portdata = pphigh: GOSUB portout
   GOSUB readack
   IF inbit = 0 THEN indat = indat + &H2000
  NEXT i
  portdata = pphigh + ppclock: GOSUB portout
  portdata = pphigh: GOSUB portout
  IF cmd = readdatadm THEN indat = indat AND 255
 END IF
ELSE
 IF cmd = programmode THEN
  portdata = 0: GOSUB portout
  GOSUB delay10ms
  portdata = ppvpp: GOSUB portout
  GOSUB delay10ms
  portdata = 0: GOSUB portout
  GOSUB delay10ms
 END IF
END IF
exitdopic:
RETURN

output1bit:
portdata = bit: GOSUB portout
portdata = bit + ppclock: GOSUB portout
portdata = bit: GOSUB portout
portdata = 0: GOSUB portout
RETURN

portout:
OUT (portadr), portdata
FOR z& = 1 TO portdelay&: NEXT
RETURN

readack:
FOR z& = 1 TO ackdelay&: NEXT
IF (INP(statadr) AND 64) = 64 THEN inbit = 1 ELSE inbit = 0
RETURN

delay10ms:
FOR z& = 1 TO tenmsdelay&: NEXT z&
RETURN

REM test delay to make sure ok, otherwise adjust
calibratedelay:
DEF SEG = &H40: t = PEEK(&H6C)
WHILE (PEEK(&H6C) = t): WEND: t = PEEK(&H6C)
FOR i = 1 TO 4: GOSUB delay10ms: NEXT i
IF t = PEEK(&H6C) THEN
 tenmsdelay& = tenmsdelay& * 1.3
 GOTO calibratedelay
END IF
RETURN

help:
PRINT "  Software for programming PICF84 with PC"
PRINT "  printer port. Sept 27 (c)1999 Terry Newton"
PRINT
PRINT "  Usage: PICPROG [/d] [/p=xxxx] [filename]"
PRINT "  /d - dump the PIC contents to a hex file"
PRINT "  /p=xxxx - specify hex port address (caution!)"
PRINT
PRINT "  Port-PIC assignments..."
PRINT "  D3 (5) -------.                     Held high when reading"
PRINT "  D2 (4) -------|--(inv)--> MCLR (4)  12v if D2=0, 0v if 1"
PRINT "  D1 (3) -------|---------> Rb6 (12)  Clock input"
PRINT "  D0 (2) -------|----*----> Rb7 (13)  Data input"
PRINT "  Ack (10) <--(inv)--'                Data output"
PRINT "  Gnd (18-25) ------------- Vss (5)   Ground"
PRINT "                     5v --- Vcc (14)  Power"
GOTO errorquit

REM for compiled... (errors if interpreted)
getparms:
f$ = LCASE$(COMMAND$)
nofilename = 1: nocl = 1: IF f$ = "" THEN RETURN
IF INSTR(f$, "/?") > 0 GOTO help
f$ = "      " + f$ + "      "
sp = INSTR(f$, "/d")
IF sp > 0 THEN
 dump = 1: f$ = LEFT$(f$, sp - 1) + RIGHT$(f$, LEN(f$) - sp - 2)
 nocl = 0
END IF
sp = INSTR(f$, "/p=")
IF sp > 0 THEN
 portadr = VAL("&H" + MID$(f$, sp + 3, 4))
 f$ = LEFT$(f$, sp - 1) + RIGHT$(f$, LEN(f$) - sp - 7)
 'if just port specified, go to menu (nocl = 1)
END IF
f$ = LTRIM$(RTRIM$(f$))' get rid of extra spaces
IF NOT f$ = "" THEN nofilename = 0: nocl = 0
RETURN

loadfile:
REM load hex file f$ into program%(), eedata%() and config%()
loaderror = 0
ON ERROR GOTO loaderrortrap
FOR i = 0 TO 1023: program%(i) = 0: NEXT i
FOR i = 0 TO 63: eedata%(i) = 0: NEXT i
FOR i = 0 TO 7: config%(i) = 0: NEXT i
OPEN f$ FOR INPUT AS #1
invaliddata = 0: nothexfile = 1
WHILE EOF(1) = 0 AND loaderror = 0
 INPUT #1, hexline$
 IF LEFT$(hexline$, 1) = ":" THEN
  stringpos = 2
  GOSUB gethexbyte: words = INT(d / 2)
  IF words > 0 THEN
   nothexfile = 0
   GOSUB getaddress  'read hi/lo into adr
   GOSUB gethexbyte  'dummy
   FOR i = 1 TO words
    GOSUB gethexword  'read lo/hi into d
    IF adr < 1024 THEN program%(adr) = d
    IF adr >= &H2100 AND adr <= &H2140 THEN eedata%(adr - &H2100) = d
    IF adr >= &H2000 AND adr <= &H2007 THEN config%(adr - &H2000) = d
    adr = adr + 1
   NEXT i
  END IF
 ELSE
  loaderror = 1: invaliddata = 1
 END IF
WEND
CLOSE
IF nothexfile THEN
 PRINT "  File is not a hex file"
 loaderror = 1
ELSE
 IF invaliddata THEN
  PRINT "  File contains invalid data"
 END IF
END IF
RETURN

loaderrortrap:
RESUME loaderr1
loaderr1:
CLOSE : loaderror = 1
RETURN

gethexbyte:
H$ = MID$(hexline$, stringpos, 2)
d = VAL("&H" + H$)
stringpos = stringpos + 2
RETURN

gethexword:
GOSUB gethexbyte: lowbyte = d
GOSUB gethexbyte: highbyte = d
d = highbyte * 256 + lowbyte
RETURN

getaddress:
GOSUB gethexbyte: highbyte = d
GOSUB gethexbyte: lowbyte = d
adr = INT((highbyte * 256 + lowbyte) / 2)
RETURN

testhardware:
hardwarefault = 0
portdata = 0: GOSUB portout
GOSUB readack: IF inbit THEN hardwarefault = 1
IF hardwarefault = 0 THEN
 portdata = pphigh: GOSUB portout
 GOSUB readack: IF inbit = 0 THEN hardwarefault = 1
END IF
RETURN

dumptofile:
portdata = ppvpp: GOSUB portout   ' make mclr low (inverted)
PRINT : PRINT "  Attach programmer to robot"
PRINT "  Press any key when done, or esc to quit"
GOSUB getkey
IF ke$ <> CHR$(27) THEN
 PRINT "  Reading PIC: "; : adrline = CSRLIN
 cmd = programmode: GOSUB dopicstuff
 FOR adr = 0 TO 1023: GOSUB dispadr
  cmd = readdata: GOSUB dopicstuff
  cmd = incaddr: GOSUB dopicstuff
  program%(adr) = indat
 NEXT adr
 FOR adr = 0 TO 63: GOSUB dispadr
  cmd = readdatadm: GOSUB dopicstuff
  cmd = incaddr: GOSUB dopicstuff
  eedata%(adr) = indat
 NEXT adr
 dat = 0: cmd = loadconfig: GOSUB dopicstuff
 FOR adr = 0 TO 7
  IF adr < 4 OR adr = 7 THEN
   GOSUB dispadr
   cmd = readdata: GOSUB dopicstuff
   config%(adr) = indat
  ELSE
   indat = 0
  END IF
  cmd = incaddr: GOSUB dopicstuff
 NEXT adr
 PRINT : PRINT "  Writing to "; f$
 GOSUB dumphex: PRINT "  Done."
END IF
GOTO quit

makeerror:
RESUME makeerr1
makeerr1:
CLOSE : PRINT "  Error writing hex file"
RETURN

dumphex:
' this is a mess, needs optimisation bigtime
ON ERROR GOTO makeerror
OPEN f$ FOR OUTPUT AS #1
diff = 8: address = 0
WHILE address < 1024
 PRINT #1, ":10"; : check = 16
 adr = address * 2: adrlow = INT(adr / 256)
 adrhigh = adr - adrlow * 256
 d = adrlow: GOSUB printhexbyte
 d = adrhigh: GOSUB printhexbyte
 check = check + adrlow: check = check + adrhigh
 PRINT #1, "00";
 FOR i = 1 TO 8
  d = program%(address) AND 255: GOSUB printhexbyte
  check = check + d
  d = INT(program%(address) / 256): GOSUB printhexbyte
  check = check + d: address = address + 1
 NEXT i
 check = check - 1
 check = check XOR &HFF
 check = check AND &HFF
 d = check: GOSUB printhexbyte: PRINT #1, ""
WEND
address = &H2000
WHILE address < &H2004
 PRINT #1, ":08"; : check = 8
 adr = address * 2: adrlow = INT(adr / 256)
 adrhigh = adr - adrlow * 256
 d = adrlow: GOSUB printhexbyte
 d = adrhigh: GOSUB printhexbyte
 check = check + adrlow: check = check + adrhigh
 PRINT #1, "00";
 FOR i = 1 TO 4
  d = config%(address - &H2000) AND 255: GOSUB printhexbyte
  check = check + d
  d = INT(config%(address - &H2000) / 256): GOSUB printhexbyte
  check = check + d: address = address + 1
 NEXT i
 check = check - 1
 check = check XOR &HFF
 check = check AND &HFF
 d = check: GOSUB printhexbyte: PRINT #1, ""
WEND
address = &H2007
PRINT #1, ":02"; : check = 2
adr = address * 2: adrlow = INT(adr / 256)
adrhigh = adr - adrlow * 256
d = adrlow: GOSUB printhexbyte
d = adrhigh: GOSUB printhexbyte
check = check + adrlow: check = check + adrhigh
PRINT #1, "00";
d = config%(7) AND 255: GOSUB printhexbyte
check = check + d
d = INT(config%(7) / 256): GOSUB printhexbyte
check = check + d: address = address + 1
check = check - 1
check = check XOR &HFF
check = check AND &HFF
d = check: GOSUB printhexbyte: PRINT #1, ""
address = &H2100
WHILE address < &H2140
 PRINT #1, ":10"; : check = 16
 adr = address * 2: adrlow = INT(adr / 256)
 adrhigh = adr - adrlow * 256
 d = adrlow: GOSUB printhexbyte
 d = adrhigh: GOSUB printhexbyte
 check = check + adrlow: check = check + adrhigh
 PRINT #1, "00";
 FOR i = 1 TO 8
  d = eedata%(address - &H2100) AND 255: GOSUB printhexbyte
  check = check + d
  d = INT(eedata%(address - &H2100) / 256): GOSUB printhexbyte
  check = check + d: address = address + 1
 NEXT i
 check = check - 1
 check = check XOR &HFF
 check = check AND &HFF
 d = check: GOSUB printhexbyte: PRINT #1, ""
WEND
PRINT #1, ":00000001FF"
CLOSE
RETURN

printhexbyte:
PRINT #1, RIGHT$("0" + HEX$(d), 2);
RETURN

